#! /usr/bin/python3
# -*- coding: utf-8 -*-

#     Copyright 2014 Olivier Schwander <olivier.schwander@ens-lyon.org>

#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.

#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.

#     You should have received a copy of the GNU General Public License
#     along with this program; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# Requirements:
# You will need pynotify and pyaml

# Usage:
# Simply launch notmuch-notify or notmuch-notify -h for usage help

"""Simple OSD notifier for notmuch"""

import os, sys, cgi, time
import argparse
import json
from subprocess import Popen, PIPE
import html

# import daemon
import notify2
import yaml

version = "0.1"

defaults = {
    "icon"    : "dialog-information",
    "title"   : "{count} unread message(s) !",
    "body"    : "<i>from</i> {authors}",
    "delay"   : 3,
    "timeout" : 3,
    "remote"  : None,
    "path"    : None,
    "authors_limit": 10,
}

def notmuch_search(query, remote=None, path=None, debug=False):
    args = []
    if remote is not None:
        args += ["ssh", remote]
    if path is not None:
        args += [path]
    else:
        args += ["notmuch"]
    
    args += ["search", "--format=json", "--output=messages"]
    args += query
    process = Popen(args, stdout=PIPE, stderr=None)
    
    if debug:
        print(args, file=sys.stderr)
    
    out = ""
    for line in process.stdout:
        out += line.decode("UTF-8")
    
    process.terminate()
    
    return json.loads(out)

def notmuch_show(messages, limit, remote=None, path=None, debug=False):
    args = []
    if remote is not None:
        args += ["ssh", remote]
    if path is not None:
        args += [path]
    else:
        args += ["notmuch"]
    
    args += ["show", "--format=json", " ".join(["id:" + m for m in messages[0:limit]])]
    if debug:
        print(args, file=sys.stderr)
    
    process = Popen(args, stdout=PIPE, stderr=None)
    out = ""
    for line in process.stdout:
        out += line.decode("UTF-8")
    
    process.terminate()
    
    return json.loads(out)

def notmuch_authors(messages, limit, remote=None, path=None, debug=False):
    content =  notmuch_show(messages, limit, remote, path, debug)
    
    authors = []
    for m in content:
        authors += m[0][0]["headers"]["From"].split(", ")
        if len(authors) > limit:
            break
    
    authors = ", ".join(authors[0:limit])
    
    return authors

class Notifier:
    """ Notifier class
    """
    _name = "notmuchtify"
    
    def notify(self):
        """ Display one notification
        """
        
        messages = notmuch_search(self._query, remote=self._remote, path=self._path, debug=self._debug)
        count    = len(messages)
        
        if count == 0:
            if self._debug:
                print("Count is 0", file=sys.stderr)
            return True
        
        authors = notmuch_authors(messages, self._authors_limit,
                                  remote=self._remote, path=self._path, debug=self._debug)
        variables = {
            "count": count,
            "authors": html.escape(authors)
        }
        title = self._title_format.format(**variables)
        body  = self._body_format.format(**variables)
        
        if self._debug:
            print("Title:", title, file=sys.stderr)
            print("Body:", body, file=sys.stderr)
        
        self.show(title, body)
    
    def show(self, title, body):
        self._notifier.update(title, body, self._icon_url)
        
        if not self._notifier.show():
            print("Notification failure", file=sys.stderr)
    
    def run(self):
        """ Display notifications regularly
        """
        while True:
            self.notify()
            # delay parameter in minutes
            time.sleep(self._delay * 60)
    
    def close(self):
        pass
    
    def __init__(self, query, config, debug):
        self._debug         = debug
        
        self._icon_url      = config["icon"]
        self._timeout       = config["timeout"]
        self._title_format  = config["title"]
        self._body_format   = config["body"]
        self._delay         = config["delay"]
        self._timeout       = config["timeout"]
        self._remote        = config["remote"]
        self._path          = config["path"]
        self._authors_limit = config["authors_limit"]
        
        self._query         = query

        self._notifier = None
        
        if not notify2.init(self._name):
            print("Failed to initialize pynotify module", file=sys.stderr)
            sys.exit(1)
        
        self._notifier = notify2.Notification("")
        
        # param timeout is in seconds
        if self._timeout == 0:
            self._notifier.set_timeout(pynotify.EXPIRES_NEVER)
        else:
            self._notifier.set_timeout(1000 * self._timeout)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Notmuchtify - Simple OSD notifier for notmuch.")
    
    parser.add_argument("-c", "--config",
                        dest="config",
                        default=os.path.expanduser("~/.config/notmuchtify/config.yml"),
                        help="Configuration file")
    parser.add_argument("--debug",
                        action="store_true",
                        dest="debug",
                        help="Turn on debugging information")
    # parser.add_argument("-d", "--daemon",
    #                     action="store_true",
    #                     dest="fork",
    #                     help="Fork into the background")
    parser.add_argument("-o", "--once",
                        action="store_true",
                        dest="once",
                        help="Notify once and exit")
    parser.add_argument("query", nargs="+")
    args = parser.parse_args()
    
    with open(args.config) as f:
        config = yaml.load(f)
    if config is None:
        config = {}
    for k, v in defaults.items():
        if k not in config:
            config[k] = v
    
    notifier = Notifier(args.query, config, args.debug)
    
    try:
        if args.once:
            notifier.notify()
        # elif args.fork and not args.debug:
        #     with daemon.DaemonContext():
        #         notifier.run()
        else:
            notifier.run()
    
    except KeyboardInterrupt:
            notifier.close()
            sys.exit(0)

